#include <proto.h>

#include "gtframe.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtFrame class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init frame.
*
\************************************************************************/
GtFrame *GtMkFrame(char *n,GtManager *p,GtTag *tgs)
{
	GtFrame *obj;
	
	if(!(obj=new GtFrame()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtFrame object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init frame.
*
\************************************************************************/
GtFrame *GtMkFrame(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtFrame *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkFrame(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty frame.
*
\************************************************************************/
GtFrame::GtFrame(void)
{
	maxchild=2;
	title=NULL;
	tw=0;
	th=0;
	ibw=0;
	bdrw=1;
	ibdrw=0;
	ftype=GTF_DBL_IN;
	talign=GTA_LEFT;
}

/************************************************************************\
*
* Setup frame.
*
\************************************************************************/
sbool GtFrame::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtBdrMan::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_FRAME,GTRT_FRAME,&ftype);
	app->GetRes(this,GTR_ALIGN,GTRT_ALIGN,&talign);
	app->GetRes(this,GTR_IBDR,GTRT_BYTE,&ibdrw);
	app->GetRes(this,GTR_BDR,GTRT_BYTE,&bdrw);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_FrameType:
				ftype=t->d.val;
				if(ftype>GTF_LAST_NR)
					ftype=GTF_DBL_IN;
				break;
			case GT_Align:
				talign=t->d.val;
				if(talign>GTA_LAST_NR)
					talign=GTA_LEFT;
				break;
			case GT_InnerBorder:
				ibdrw=t->d.val;
				break;
			case GT_Border:
				bdrw=t->d.val;
				break;
		}
	}
	switch(ftype)
	{
		case GTF_NONE:
			ibw=ibdrw;
			break;
		case GTF_SIMPLE:
		case GTF_IN:
		case GTF_OUT:
			ibw=ibdrw+bdrw;
			break;
		case GTF_DBL_IN:
		case GTF_DBL_OUT:
			ibw=ibdrw+2*bdrw;
			break;
	}
	return(TRUE);
}

/************************************************************************\
*
* Set frame attributes.
*
\************************************************************************/
sbool GtFrame::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE,ach=FALSE;
	
	if(!GtBdrMan::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_FrameType:
				ftype=t->d.val;
				if(ftype>GTF_LAST_NR)
					ftype=GTF_DBL_IN;
				ch=TRUE;
				break;
			case GT_Align:
				talign=t->d.val;
				if(talign>GTA_LAST_NR)
					talign=GTA_LEFT;
				ach=TRUE;
				break;
			case GT_InnerBorder:
				ibdrw=t->d.val;
				ch=TRUE;
				break;
			case GT_Border:
				bdrw=t->d.val;
				ch=TRUE;
				break;
		}
	}
	if(ch || ach)
	{
		switch(ftype)
		{
			case GTF_NONE:
				ibw=ibdrw;
				break;
			case GTF_SIMPLE:
			case GTF_IN:
			case GTF_OUT:
				ibw=ibdrw+bdrw;
				break;
			case GTF_DBL_IN:
			case GTF_DBL_OUT:
				ibw=ibdrw+2*bdrw;
				break;
		}
		if(ach)
			this->Relayout();
		this->Refresh();
	}
	return(TRUE);
}

/************************************************************************\
*
* Get frame attributes.
*
\************************************************************************/
void GtFrame::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtBdrMan::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_FrameType:
				*ptr=ftype;
				break;
			case GT_Align:
				*ptr=talign;
				break;
			case GT_InnerBorder:
				*ptr=ibdrw;
				break;
			case GT_Border:
				*ptr=bdrw;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtFrame::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtFrame")==0)
		return(TRUE);
	else
		return(GtBdrMan::IsSubClassOf(cn));
}

/************************************************************************\
*
* Add child and make sure that the title widget is last.
*
\************************************************************************/
sbool GtFrame::AddChild(GtWidget *obj,ulong pos)
{
	pos=pos;
	if(title)
		return(GtBdrMan::AddChild(obj,0));
	else
		return(GtBdrMan::AddChild(obj,1));
}

/************************************************************************\
*
* Remove child from frame so check if it was the title.
*
\************************************************************************/
void GtFrame::RemChild(GtWidget *obj)
{
	GtBdrMan::RemChild(obj);
	if(obj==title)
		title=NULL;
}

/************************************************************************\
*
* Relayout frame with title.
*
\************************************************************************/
void GtFrame::Relayout(void)
{
	GtWListBdr *li;
	GtWidget *obj=NULL;
	int xs,ys;
	word mxs=1,mys=1,mxxs=GTS_MAX,mxys=GTS_MAX;
	
	if(flags & GTWF_UNBUILT)
		return;
	if((li=(GtWListBdr *)GtSFirst(&man)) && li->wl.w->IsMapped())
		obj=li->wl.w;
	if(title)
	{
		li=(GtWListBdr *)title->GetLInfo();
		title->GetMinSize(&xs,&ys);
		tw=GTMAX(xs,li->minxs)+li->bleft+li->bright;
		th=GTMAX(ys,li->minys)+li->btop+li->bbot;
		if(obj)
		{
			li=(GtWListBdr *)obj->GetLInfo();
			obj->GetMinSize(&xs,&ys);
			mxs=GTMAX(2*ibw+GTMAX(xs,li->minxs)+li->bleft+li->bright,2*ibw+4+tw);
			mys=GTMAX(ibdrw+ibw+GTMAX(th,ibw)+GTMAX(ys,li->minys)+li->btop+li->bbot,
									ibdrw+ibw+GTMAX(th,ibw));
			obj->GetMaxSize(&xs,&ys);
			mxxs=2*ibw+GTMIN(xs,li->maxxs)+li->bleft+li->bright;
			mxys=ibdrw+ibw+GTMAX(th,bdrw)+GTMIN(ys,li->maxys)+li->btop+li->bbot;
		}
		else
		{
			mxs=2*ibw+4+tw;
			mys=ibw+th;
			mxxs=GTS_MAX;
			mxys=GTS_MAX;
		}
	}
	else
	{
		tw=0;
		th=0;
		if(obj)
		{
			li=(GtWListBdr *)obj->GetLInfo();
			obj->GetMinSize(&xs,&ys);
			mxs=GTMAX(2*ibw+GTMAX(xs,li->minxs)+li->bleft+li->bright,2*ibw);
			mys=GTMAX(2*ibw+GTMAX(ys,li->minys)+li->btop+li->bbot,2*ibw);
			obj->GetMaxSize(&xs,&ys);
			mxxs=2*ibw+GTMIN(xs,li->maxxs)+li->bleft+li->bright;
			mxys=2*ibw+GTMIN(ys,li->maxys)+li->btop+li->bbot;
		}
	}
	this->SetMinMaxSize(GTMAX(mxs,2*ibw+1),GTMAX(mys,2*ibw+1),mxxs,mxys);
}

/************************************************************************\
*
* Set child layout information attributes. Check if child is title.
*
\************************************************************************/
sbool GtFrame::SetLInfo(GtWidget *obj,GtTag *tgs)
{
	long l;
	sbool res=FALSE;
	
	if(!title)
	{
		l=GtGetTagVal(tgs,GT_FrameTitle,FALSE);
		if(l)
		{
			title=(GtWidget *)obj;
			res=TRUE;
		}
	}
	return(GtBdrMan::SetLInfo(obj,tgs) || res);
}
	
/************************************************************************\
*
* Set frame event mask.
*
\************************************************************************/
void GtFrame::SetEMask(ulong em)
{
	GtBdrMan::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Layout frame and title.
*
\************************************************************************/
void GtFrame::Layout(void)
{
	GtWidget *obj=NULL;
	GtWListBdr *li;
	int xp=0,yp=0;
	
	if(flags & GTWF_UNBUILT)
		return;
	if((li=(GtWListBdr *)GtSFirst(&man)) && li->wl.w->IsMapped())
		obj=li->wl.w;
	if(obj)
	{
		if(flags & GTWF_NOWIN)
		{
			xp=xpos;
			yp=ypos;
		}
		obj->SetPosSize(xp+ibw+li->bleft,yp+(th ? ibdrw+GTMAX(th,ibw) : ibw)+li->btop,
			(GTMIN(GTMAX(xsize,minxs),maxxs)-2*ibw-li->bleft-li->bright), 
			(GTMIN(GTMAX(ysize,minys),maxys)-(th ? ibdrw+ibw+GTMAX(th,bdrw) : 2*ibw)-
				li->btop-li->bbot));
	}
	if(title)
	{
		li=(GtWListBdr *)title->GetLInfo();
		if(bdrw>th)
			yp=(bdrw-th)/2;
		else
			yp=0;
		switch(talign)
		{
			case GTA_LEFT:
				xp=ibw+2;
				break;
			case GTA_RIGHT:
				xp=xsize-ibw-2-tw;
				break;
			case GTA_CENTER:
				xp=2+(xsize-2*ibw-4-tw)/2;
				break;
		}
		if(flags & GTWF_NOWIN)
		{
			xp+=xpos;
			yp+=ypos;
		}
		title->SetPosSize(xp,yp,tw,th);
		tx=xp;
		ty=yp;
	}
}

/************************************************************************\
*
* Draw frame.
*
\************************************************************************/
sbool GtFrame::Draw(XEvent *xe)
{
	int n,xpe,ype,yp=0;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(bdrw>th)
		yp=0;
	else
		yp=th/2;
	if(flags & GTWF_NOWIN)
	{
		xpe=xpos;
		ype=ypos;
	}
	else
	{
		xpe=0;
		ype=0;
	}
	switch(ftype)
	{
		case GTF_SIMPLE:
			for(n=0; n<bdrw; n++)
				gtfore->DrawRectangle(gtwin,xpe+n,ype+n+yp,xsize-2*n-1,ysize-2*n-yp-1,FALSE);
			break;
		case GTF_IN:
			for(n=0; n<bdrw; n++)
			{
				GT3DBOX(gtdrk,gtlht,gtwin,xpe+n,ype+n+yp,xsize-2*n,ysize-2*n-yp);
			}
			break;
		case GTF_OUT:
			for(n=0; n<bdrw; n++)
			{
				GT3DBOX(gtlht,gtdrk,gtwin,xpe+n,ype+n+yp,xsize-2*n,ysize-2*n-yp);
			}
			break;
		case GTF_DBL_IN:
			for(n=0; n<bdrw; n++)
			{
				GT3DBOX(gtdrk,gtlht,gtwin,xpe+n,ype+n+yp,xsize-2*n,ysize-2*n-yp);
				GT3DBOX(gtlht,gtdrk,gtwin,xpe+bdrw+n,ype+bdrw+n+yp,
								xsize-2*(bdrw+n),ysize-2*(bdrw+n)-yp);
			}
			break;
		case GTF_DBL_OUT:
			for(n=0; n<bdrw; n++)
			{
				GT3DBOX(gtlht,gtdrk,gtwin,xpe+n,ype+n+yp,xsize-2*n,ysize-2*n-yp);
				GT3DBOX(gtdrk,gtlht,gtwin,xpe+bdrw+n,ype+bdrw+n+yp,
								xsize-2*(bdrw+n),ysize-2*(bdrw+n)-yp);
			}
			break;
	}
	if(title && (flags & GTWF_NOWIN))
		gtwin->Clear(tx,ty,tw,th);
	return(TRUE);
}
